"""Describe tkinter button with default common args"""
import tkinter as tk
import tkinter.ttk as ttk
from customtkinter import *
from pollenisatorgui.core.forms.form import Form


class FormButton(Form):
    """
    Form field representing a button.
    Default setted values: 
        if pack : padx = pady = 5, side = right
        if grid: row = column = 0 sticky = "west
    """

    def __init__(self, name, callback, **kwargs):
        """
        Constructor for a form button

        Args:
            name: the button text.
            callback: a function that will be called when the button is clicked.
            kwargs: same keyword args as you would give to CTkButton
        """
        super().__init__(name)
        self.callback = callback
        self.kwargs = kwargs
        self.btn = None
        self.wid_kwargs = None
        self.infos = {}

    def callback_infos(self, event):
        self.callback(event, self.infos)

    def constructView(self, parent):
        """
        Create the button view inside the parent view given

        Args:
            parent: parent form panel.
        """
        s = self.getKw("style", None)
        text = self.getKw("text", self.name)
        tooltip = self.getKw("tooltip", None)
        if s is None and tooltip is None:
            self.btn = CTkButton(parent.panel, text=text, width=int(self.getKw("width", 150)), height=int(self.getKw("height", 28)), image=self.getKw("image", None),
                                  border_color=self.getKw("border_color", None),
                                  border_width=self.getKw("border_width", None),
                                  text_color=self.getKw("text_color", None),
                                  hover_color=self.getKw("hover_color", None),
                                  fg_color=self.getKw("fg_color", None), font=self.getKw("font", None),
                                  state=self.getKw("state", "normal"))
        else:
            self.btn = ttk.Button(parent.panel, text=text, image=self.getKw("image", None), style=s)
            if tooltip is not None:
                self.btn.configure(tooltip=tooltip)
        self.infos = self.getKw("infos", {})
        if len(self.infos) > 0:
            self.btn.bind('<Button-1>', self.callback_infos)
        else:
            self.btn.bind('<Button-1>', self.callback)
        for bind, bind_call in self.getKw("binds", {}).items():
            self.btn.bind(bind, bind_call)
        
        if parent.gridLayout:
            self.btn.grid(row=self.getKw("row", 0), column=self.getKw("column", 0), sticky=self.getKw("sticky", tk.W), **self.kwargs)
        else:
            self.btn.pack(side=self.getKw("side", "right"), padx=self.getKw("padx", 5), pady=self.getKw("pady", 5), **self.kwargs)
        if self.wid_kwargs is not None:
            self.btn.configure(**self.wid_kwargs)

    def configure(self, **kwargs):
        """Change kwargs to given one. Must be called before constructView
        Args:
            **kwargs: any ttk Button keyword arguments."""
        if self.btn is None:
            self.wid_kwargs = kwargs
        else:
            self.btn.configure(**kwargs)
  

    def setFocus(self):
        """Set the focus to the ttk button.
        """
        self.btn.focus_set()


"""View for remark object. Handle node in treeview and present forms to user when interacted with."""

from tkinter import TclError
import tkinter as tk
from pollenisatorgui.core.views.viewelement import ViewElement
from pollenisatorgui.core.models.remark import Remark
from pollenisatorgui.core.components.apiclient import APIClient
import pollenisatorgui.core.components.utils as utils
from shutil import which
import os
import sys
import subprocess



class RemarkView(ViewElement):
    """View for remark object. Handle node in treeview and present forms to user when interacted with.
    Attributes:
        icon: icon name to show in treeview. Icon filename must be in icon directory.
    """

    icon = 'defects.png'

    cached_icons =  {}

    @classmethod
    def getIconName(cls, typeOfRemark):
        return "remark_"+typeOfRemark.lower()+".png"

    @classmethod
    def getIcon(cls, typeOfRemark):
        """
        Load the object icon in cache if it is not yet done, and returns it

        Return:
            Returns the icon representing this object.
        """
        cache = cls.cached_icons.get(typeOfRemark, None)

        if cache is None:
            from PIL import Image, ImageTk
            path = utils.getIcon(cls.getIconName(typeOfRemark))
            cls.cached_icons[typeOfRemark] = ImageTk.PhotoImage(Image.open(path))
            return cls.cached_icons[typeOfRemark]
        return cache

    def __init__(self, appViewFrame, controller):
        """Constructor
        Args:
            controller: a CommandController for this view.
        """
        super().__init__(None, appViewFrame, None, controller)
        self.comboTypeForm = None
        self.imgTypeForm = None

    def searchCallback(self, searchreq, **options):
        remarks_obj, remarks_errors = APIClient.getInstance().searchRemark(searchreq, **options)
        if remarks_obj:
            for i, remark in enumerate(remarks_obj):
                remarks_obj[i]["TITLE"] = remark["title"]
        return remarks_obj, remarks_errors
    
    def openInsertWindow(self,  **kwargs):
        """
        Creates a tkinter form using Forms classes. This form aims to insert a new Remark
        Args:
            addButtons: boolean value indicating that insertion buttons should be visible. Default to True
        """
        modelData = self.controller.getData()
        
        search_panel = self.form.addFormPanel(grid=True, side="left", pady=0, padx=0)
        type_combo_search = search_panel.addFormPanel(grid=True, row=0, column=0)
        type_combo_search.addFormLabel("Type for search",  row=0, column=0)
        type_search = type_combo_search.addFormCombo("Search only", ["<Empty>", "Positive","Neutral","Negative"], row=0, column=1)
        s = search_panel.addFormSearchBar("Search Remark", self.searchCallback, None, row=1)
        s.addOptionForm(type_search, "remark_type")
        
        topPanel = self.form.addFormPanel(grid=True, side="right")
        head = topPanel.addFormPanel(grid=True, row=0)
        self.imgTypeForm = head.addFormImage(utils.getIcon(RemarkView.getIconName(modelData["type"])))
        self.comboTypeForm = head.addFormCombo("Type", ["Positive","Neutral","Negative"], command=self.updateImage, column=1, default=modelData["type"], binds={"<<ComboboxSelected>>": self.updateImage, "<<FormUpdated>>": self.updateImage})
        self.comboTypeForm.configure(command=self.updateImage)
        head.addFormStr("Title", r".+", "", column=2, placeholder_text="Title")
        bot = topPanel.addFormPanel(grid=True,row=1)
        bot.addFormText("Description", r".+", "description")
        s.panel_to_fill = topPanel
        if kwargs.get("addButtons", True):
            self.completeInsertWindow()
        else:
            self.showForm()

    def openModifyWindow(self, **kwargs):
        """
        Creates a tkinter form using Forms classes.
        This form aims to update or delete an existing Defect
        Args:
            addButtons: boolean value indicating that insertion buttons should be visible. Default to True
        """
        self.form.clear()
        modelData = self.controller.getData()
        search_panel = self.form.addFormPanel(grid=True, side="left", pady=0, padx=0)
        type_combo_search = search_panel.addFormPanel(grid=True, row=0, column=0)
        type_combo_search.addFormLabel("Type for search",  row=0, column=0)
        type_search = type_combo_search.addFormCombo("Search only", ["<Empty>", "Positive","Neutral","Negative"], row=0, column=1)
        s = search_panel.addFormSearchBar("Search Remark", self.searchCallback, self.form, row=1)

        topPanel = self.form.addFormPanel(grid=True, side="right")
        self.imgTypeForm = topPanel.addFormImage(utils.getIcon(RemarkView.getIconName(modelData["type"])))
        self.comboTypeForm = topPanel.addFormCombo("Type", ["Positive","Neutral","Negative"], column=1, default=modelData["type"],command=self.updateImage, binds={"<<ComboboxSelected>>": self.updateImage, "<<FormUpdated>>": self.updateImage})
        self.comboTypeForm.configure(command=self.updateImage)
        s.addOptionForm(type_search, "remark_type")
        topPanel.addFormStr(
            "Title", r".+", modelData["title"], column=2)
        topPanel.addFormText("Description", r".+", modelData["description"], row=2,column=2)
        if kwargs.get("addButtons", True):
            self.completeModifyWindow()
        else:
            self.showForm()

    def updateImage(self, _event=None):
        """Callback when ease or impact is modified.
        Calculate new resulting risk value
        Args
            _event: mandatory but not used
        """
        typeof = self.comboTypeForm.getValue()
        self.imgTypeForm.setImage(utils.getIcon(RemarkView.getIconName(typeof)))


    


import multiprocessing
import subprocess
import psutil
from pollenisatorgui.core.components.logger_config import logger
import pollenisatorgui.core.components.utils as utils
from pollenisatorgui.core.application.dialogs.ChildDialogQuestion import ChildDialogQuestion
from pollenisatorgui.core.application.dialogs.ChildDialogCombo import ChildDialogCombo
import tempfile
import os
import shutil
import tkinter as tk
from pollenisatorgui.core.components.apiclient import APIClient
from pollenisatorgui.scripts.lan.utils import checkPath, findDc


def main(apiclient, appli, **kwargs):
    res, path = checkPath(["ntlmrelayx.py", "ntlmrelayx"])
    if not res:
        return False, path
    APIClient.setInstance(apiclient)
    res, dc = findDc(apiclient)
    if not res:
        return False, dc
    addrs = psutil.net_if_addrs()
    dialog = ChildDialogCombo(None, addrs.keys(), displayMsg="Choose your ethernet device to listen on")
    dialog.app.wait_window(dialog.app)
    if dialog.rvalue is None:
        return False, "No ethernet device chosen"
    my_ip = addrs[dialog.rvalue][0].address
    cmd = f"{path} -t ldap://{dc} -smb2support -wh {my_ip} -6" 
    if os.geteuid() != 0:
        cmd = "sudo "+cmd
    appli.launch_in_terminal(kwargs.get("default_target",None), "ntlmrelayx to ldapr", cmd, use_pollex=False)
    return True, f"Relaying is set up, poison the network using responder, mitm6, arp spoofing, etc."



# coding: utf-8


class ComputerInfos():
    def __init__(self, valuesFromDb=None):
        """
        :param os: The os of this ComputerInfos.
        :type os: str
        :param signing: The signing of this ComputerInfos.
        :type signing: boolean
        :param smbv1: The smbv1 of this ComputerInfos.
        :type smbv1: boolean
        :param is_dc: The is_dc of this ComputerInfos.
        :type is_dc: boolean
        :param secrets: The secrets of this ComputerInfos.
        :type secrets: List[str]
        """
        if valuesFromDb is None:
            valuesFromDb = {}
        self.initialize(valuesFromDb.get("os"), valuesFromDb.get("signing"),valuesFromDb.get("smbv1"), \
             valuesFromDb.get("is_dc"), valuesFromDb.get("secrets"))
        

    def initialize(self, os=None, signing=None, smbv1=None, is_dc=None, secrets=None): 
        self.os = os
        self.signing = signing
        self.smbv1 = smbv1
        self.is_dc = is_dc
        self.secrets = secrets
        return self

    def getData(self):
        return {"os":self.os, "signing": self.signing, "smbv1":self.smbv1, "is_dc":self.is_dc, "secrets":self.secrets}
    
    @property
    def os(self):
        """Gets the os of this ComputerInfos.


        :return: The os of this ComputerInfos.
        :rtype: str
        """
        return self._os

    @os.setter
    def os(self, os):
        """Sets the os of this ComputerInfos.


        :param os: The os of this ComputerInfos.
        :type os: str
        """

        self._os = os

    @property
    def signing(self):
        """Gets the signing of this ComputerInfos.


        :return: The signing of this ComputerInfos.
        :rtype: bool
        """
        return self._signing

    @signing.setter
    def signing(self, signing):
        """Sets the signing of this ComputerInfos.


        :param signing: The signing of this ComputerInfos.
        :type signing: bool
        """

        self._signing = signing

    @property
    def smbv1(self):
        """Gets the smbv1 of this ComputerInfos.


        :return: The smbv1 of this ComputerInfos.
        :rtype: bool
        """
        return self._smbv1

    @smbv1.setter
    def smbv1(self, smbv1):
        """Sets the smbv1 of this ComputerInfos.


        :param smbv1: The smbv1 of this ComputerInfos.
        :type smbv1: bool
        """

        self._smbv1 = smbv1

    @property
    def is_dc(self):
        """Gets the is_dc of this ComputerInfos.


        :return: The is_dc of this ComputerInfos.
        :rtype: bool
        """
        return self._is_dc

    @is_dc.setter
    def is_dc(self, is_dc):
        """Sets the is_dc of this ComputerInfos.


        :param is_dc: The is_dc of this ComputerInfos.
        :type is_dc: bool
        """

        self._is_dc = is_dc

    @property
    def secrets(self):
        """Gets the secrets of this ComputerInfos.


        :return: The secrets of this ComputerInfos.
        :rtype: List[str]
        """
        return self._secrets

    @secrets.setter
    def secrets(self, secrets):
        """Sets the secrets of this ComputerInfos.


        :param secrets: The secrets of this ComputerInfos.
        :type secrets: List[str]
        """

        self._secrets = secrets


"""Display a simple information for the user.
"""
import tkinter as tk
import tkinter.ttk as ttk
from customtkinter import *


class ChildDialogInfo:
    """
    Open a child dialog of a tkinter application to inform the user.
    """

    def __init__(self, parent, title, msg):
        """
        Open a child dialog of a tkinter application to choose autoscan settings.

        Args:
            parent: the tkinter parent view to use for this window construction.
            title: title of the popup window
            msg: Message to show to the user
        """
        self.app = CTkToplevel(parent)
        self.app.attributes("-type", "dialog")
        self.app.resizable(False, False)
        self.app.title(title)
        self.app.bind("<Escape>", self.destroy)
        appFrame = CTkFrame(self.app)
        self.rvalue = None
        self.parent = parent
        lbl = CTkLabel(appFrame, text=msg)
        lbl.pack(side=tk.TOP, padx=10, pady=10, fill=tk.X)
        appFrame.pack(fill=tk.BOTH)
        try:
            self.app.wait_visibility()
            self.app.transient(parent)
            self.app.focus_force()
            #self.app.grab_set()
            self.app.lift()
        except tk.TclError:
            pass

    def show(self):
        """Start displaying this window."""
        self.app.update()

    def destroy(self, _event=None):
        """
        Close the window.
        """
        # send the data to the parent
        self.rvalue = None
        self.app.destroy()



"""Shared notes module"""
import tkinter as tk
import tkinter.ttk as ttk
from customtkinter import *
from pollenisatorgui.core.components.apiclient import APIClient
from pollenisatorgui.modules.module import Module
from pollenisatorgui.core.components.settings import Settings
from pollenisatorgui.core.application.mdframe import TkintermdFrame
import threading
from difflib import Differ


class SharedNotes(Module):
    """
    Shared notes tab. 
    """
    iconName = "notes.png"
    tabName = "Notes"
    coll_name = "documents"

    order_priority = Module.MEDIUM_PRIORITY
    
    def __init__(self, parent, settings, tkApp):
        """
        Constructor
        """
        super().__init__()
        self.parent = None
        self.tkApp = tkApp
        self.inited = False
        self.settings = settings

    def open(self, view, nbk, treevw):
        apiclient = APIClient.getInstance()
        self.treevw = treevw
        if not self.inited:
            self.initUI(view)
        if apiclient.getCurrentPentest() is not None:
            self.refreshUI()
        self.timer = None
        @self.tkApp.sio.on("load-document")
        def load_document(document):
            self.setText(document)
            self.save_document()
        apiclient = APIClient.getInstance()
        pentest = apiclient.getCurrentPentest()
        self.tkApp.sio.emit("get-document", {"doc_id":pentest, "pentest":pentest})
        @self.tkApp.sio.on("received-delta")
        def handler(self, delta):
            print(delta)

    def refreshUI(self):
        """
        Reload data and display them
        """
        pass

    def setText(self, text):
        self.mdFrame.text_area.delete(1.0, tk.END)
        if isinstance(text, dict) and len(text) == 0:
            text = ""
        self.mdFrame.text_area.insert(tk.INSERT, text)
 
    def initUI(self, parent):
        """
        Initialize Dashboard widgets
        Args:
            parent: its parent widget
        """
        self.inited = True
        self.parent = parent
       
        dark_mode = self.settings.is_dark_mode()
        self.mdFrame = TkintermdFrame(parent, default_text="", tkApp=self.tkApp, just_editor=False, style_change=False, enable_preview=True)
        if dark_mode:
            self.mdFrame.load_style("material")
        else:
            self.mdFrame.load_style("stata-dark")
        self.sv = StringVar()
        self.mdFrame.text_area.bind("<<KeyRelease>>", self.on_input_change)

        self.mdFrame.pack(fill="both", expand=1)
        parent.pack(fill="both", expand=1)
        #self.moduleFrame.pack(padx=10, pady=10, side=tk.TOP, fill=tk.BOTH, expand=True)


    def on_input_change(self, event):
        old_text = self.sv.get()
        new_text = self.mdFrame.text_area.get("1.0", END)
        difference = []
        for d in Differ().compare(old_text, new_text):
            difference.append(d)
        self.sv.set(new_text)
        self.tkApp.sio.emit("send-delta", difference)

    def save_document(self):
        # tell to the main thread to save the document
        try:
            self.tkApp.after(0, self.do_save_document)
        except RuntimeError:
            return
        self.timer = threading.Timer(0.2, self.save_document)
        self.timer.start()

    def do_save_document(self):
        self.tkApp.sio.emit("save-document", self.mdFrame.text_area.get(1.0, tk.END))

    def close(self):
        self.tkApp.sio.on("received-delta",  )
        self.timer.cancel()


"""View for tool object. Handle node in treeview and present forms to user when interacted with."""

from pollenisatorgui.core.application.CollapsibleFrame import CollapsibleFrame
from pollenisatorgui.core.components.datamanager import DataManager
from pollenisatorgui.core.forms.formpanel import FormPanel
from pollenisatorgui.core.views.viewelement import ViewElement
from pollenisatorgui.core.components.apiclient import APIClient
import tkinter.messagebox
import tkinter as tk
from tkinter import TclError
from pollenisatorgui.core.views.defectview import DefectView
from pollenisatorgui.core.models.defect import Defect
from pollenisatorgui.core.controllers.defectcontroller import DefectController
from pollenisatorgui.core.application.dialogs.ChildDialogQuestion import ChildDialogQuestion
from pollenisatorgui.core.application.dialogs.ChildDialogInfo import ChildDialogInfo
import pollenisatorgui.core.components.utils as utils
from bson import ObjectId, errors
from customtkinter import *
import os
import json
from PIL import Image

class ToolView(ViewElement):
    """View for tool object. Handle node in treeview and present forms to user when interacted with.
    Attributes:
        icon: icon name to show in treeview. Icon filename must be in icon directory
        done_icon: icon filename for done tools
        ready_icon: icon filename for ready tools
        running_icon: icon filename for running tools
        not_ready_icon: icon filename for not ready tools
        cached_icon: a cached loaded PIL image icon of ToolView.icon. Starts as None.
        cached_done_icon: a cached loaded PIL image icon of ToolView.done_icon. Starts as None.
        cached_ready_icon: a cached loaded PIL image icon of ToolView.ready_icon. Starts as None.
        cached_running_icon: a cached loaded PIL image icon of ToolView.running_icon. Starts as None.
        cached_not_ready_icon: a cached loaded PIL image icon of ToolView.not_ready_icon. Starts as None.
        """

    done_icon = 'done_tool.png'
    error_icon = 'error_tool.png'
    ready_icon = 'waiting.png'
    running_icon = 'running.png'
    not_ready_icon = 'cross.png'
    icon = 'tool.png'


    cached_icon = None
    cached_done_icon = None
    cached_ready_icon = None
    cached_running_icon = None
    cached_not_ready_icon = None
    cached_error_icon = None

    def getIcon(self):
        """
        Load the object icon in cache if it is not yet done, and returns it

        Return:
            Returns the icon representing this object.
        """
        status = self.controller.getStatus()
        iconStatus = "ready"
        if "done" in status:
            cache = self.__class__.cached_done_icon
            ui = self.__class__.done_icon
            iconStatus = "done"
        elif "running" in status:
            ui = self.__class__.running_icon
            cache = self.__class__.cached_running_icon
            iconStatus = "running"
        elif "error" in status or "timedout" in status:
            cache = self.__class__.cached_error_icon
            ui = self.__class__.error_icon
            iconStatus = "error"
        elif "OOS" not in status and "OOT" not in status:
            ui = self.__class__.ready_icon
            cache = self.__class__.cached_ready_icon
            iconStatus = "ready"
        else:
            ui = self.__class__.not_ready_icon
            cache = self.__class__.cached_not_ready_icon
        # FIXME, always get triggered
        # if status == [] or iconStatus not in status :
        #     print("status is "+str(status)+ " or "+str(iconStatus)+" not in "+str(status))
        #     self.controller.setStatus([iconStatus])

        if cache is None:
            from PIL import Image, ImageTk
            path = utils.getIcon(ui)
            if iconStatus == "done":
                self.__class__.cached_done_icon = ImageTk.PhotoImage(
                    Image.open(path))
                return self.__class__.cached_done_icon
            elif iconStatus == "error":
                self.__class__.cached_error_icon = ImageTk.PhotoImage(
                    Image.open(path))
                return self.__class__.cached_error_icon
            elif iconStatus == "running":
                self.__class__.cached_running_icon = ImageTk.PhotoImage(
                    Image.open(path))
                return self.__class__.cached_running_icon
            elif iconStatus == "ready":
                self.__class__.cached_ready_icon = ImageTk.PhotoImage(
                    Image.open(path))
                return self.__class__.cached_ready_icon
            else:
                self.__class__.cached_not_ready_icon = ImageTk.PhotoImage(
                    Image.open(path))
                return self.__class__.cached_not_ready_icon
        return cache

    

    def openModifyWindow(self, **kwargs):
        """
        Creates a tkinter form using Forms classes. This form aims to update or delete an existing Tool
        """
        self.form.clear()
        modelData = self.controller.getData()
        
        top_panel = self.form.addFormPanel()
        top_panel.addFormLabel("Command executed")
        top_panel.addFormStr("Command executed", "", modelData.get("infos", {}).get("cmdline",""), width=200)
        notes = modelData.get("notes", "")
        top_panel = self.form.addFormPanel()
        top_panel.addFormLabel("Notes", side="top")
        top_panel.addFormText("Notes", r"", notes, None, height=500, side="top")
        
        actions_panel = self.form.addFormPanel()
        apiclient = APIClient.getInstance()
        datamanager = DataManager.getInstance()
        self.image_download = CTkImage(Image.open(utils.getIcon("download.png")))
        self.image_reset = CTkImage(Image.open(utils.getIcon("reset_small.png")))
        try:
            command_d = datamanager.get("commands", modelData["command_iid"])
            if command_d is not None: # command not registered, I.E import
                workers = apiclient.getWorkers({"pentest":apiclient.getCurrentPentest(), "known_commands":command_d.bin_path})
                workers = [x["name"] for x in workers]
                hasWorkers = len(workers)
            #Ready is legacy, OOS and/or OOT should be used
            if ("ready" in self.controller.getStatus() or "error" in self.controller.getStatus() or "timedout" in self.controller.getStatus()) or len(self.controller.getStatus()) == 0:
                if apiclient.getUser() in command_d.owners:
                    actions_panel.addFormButton(
                        "Local launch", self.localLaunchCallback, side="right")
                if any(x in command_d.owners for x in workers):
                    actions_panel.addFormButton(
                        "Run on worker", self.launchCallback, side="right")
                elif apiclient.getUser() not in command_d.owners:
                    actions_panel.addFormLabel(
                        "Info", "Tool is ready", side="right")
            elif "OOS" in self.controller.getStatus() or "OOT" in self.controller.getStatus():
                actions_panel.addFormButton(
                    "Local launch", self.localLaunchCallback, side="right")
                if hasWorkers:
                    actions_panel.addFormButton(
                        "Run on worker", self.launchCallback, side="right")
                else:
                    actions_panel.addFormLabel(
                        "Info", "Tool is ready but no worker found", side="right")
            elif "running" in self.controller.getStatus():
                actions_panel.addFormButton(
                    "Stop", self.stopCallback, side="right")
                #actions_panel.addFormButton(
                #    "Remote Interaction", self.remoteInteraction, side="right")
            elif "done" in self.controller.getStatus():
                actions_panel.addFormButton(
                    "Download result file", self.downloadResultFile, image=self.image_download, side="right")
                try:
                    mod = utils.loadPlugin(self.controller.model.getCommand()["plugin"])
                    pluginActions = mod.getActions(self.controller.model)
                except KeyError:  # Happens when parsed an existing file.:
                    pluginActions = None
                except Exception:
                    pluginActions = None
                if pluginActions is not None:
                    for pluginAction in pluginActions:
                        actions_panel.addFormButton(
                            pluginAction, pluginActions[pluginAction], side="right")
                    if command_d is not None:
                        actions_panel.addFormButton(
                            "Reset", self.resetCallback, side="right", image=self.image_reset, fg_color=utils.getBackgroundColor(), text_color=utils.getTextColor(),
                               border_width=1, border_color="firebrick1", hover_color="tomato")
        except errors.InvalidId:
            pass
        dates_panel = self.form.addFormCollapsiblePanel(text="Show more details", interior_padx=4, interior_pady=15 , grid=True, fill="both")
        dates_panel.addFormLabel("Name", modelData["name"],row=0)
        dates_panel.addFormLabel("Start date", row=1, column=0)
        dates_panel.addFormDate(
            "Start date", self.mainApp, modelData["dated"], row=1, column=1)
        dates_panel.addFormLabel("End date", row=2)
        dates_panel.addFormDate(
            "End date", self.mainApp, modelData["datef"], row=2, column=1)
        dates_panel.addFormLabel("Scanner", row=3)
        dates_panel.addFormStr(
            "Scanner", r"", modelData["scanner_ip"], row=3, column=1)
        dates_panel.addFormLabel("Infos",  row=4)
        dates_panel.addFormText("Infos", utils.is_json, json.dumps(modelData["infos"], indent=4, cls=utils.JSONEncoder), height=100, row=4, column=1)
        defect_panel = self.form.addFormPanel(grid=True)
        defect_panel.addFormButton("Create defect", self.createDefectCallback, padx=5)
        defect_panel.addFormButton("Show associated command", self.showAssociatedCommand, column=1)

        self.completeModifyWindow()

    def addInTreeview(self, parentNode=None, **kwargs):
        """Add this view in treeview. Also stores infos in application treeview.
        Args:
            parentNode: if None, will calculate the parent. If setted, forces the node to be inserted inside given parentNode.
            _addChildren: not used for tools

        """
        
        mustHide = False
        if parentNode is None:
            mustHide = True 
            parentNode = self.controller.getParentId()
            if parentNode is None:
                parentNode = ""

        nodeText = str(self.controller.getModelRepr())
        self.appliTw.views[str(self.controller.getDbId())] = {"view": self}
        if kwargs.get("insert_parents", False) and parentNode != "" and parentNode is not None:
            try:
                self.appliTw.insertViewById(parentNode)
                mustHide = False
            except tk.TclError as e:
                pass
        try:
            self.appliTw.insert(parentNode, "end", str(
                self.controller.getDbId()), text=nodeText, tags=self.controller.getTags(), image=self.getIcon())
        except tk.TclError as e:
            pass
        if self.mainApp.settings.is_checklist_view():
            self.hide("checklist_view")
        if "hidden" in self.controller.getTags() or mustHide:
            self.hide("tags")

    def downloadResultFile(self, _event=None):
        """Callback for tool click 
        Download the tool result file and asks the user if he or she wants to open it. 
        If OK, tries to open it
        Args:
            _event: not used 
        """
        apiclient = APIClient.getInstance()
        dialog = ChildDialogInfo(
            self.appliViewFrame, "Download Started", "Downloading...")
        dialog.show()
        abs_path = os.path.dirname(os.path.abspath(__file__))
        outputDir = os.path.join(abs_path, "../../results")
        path = self.controller.getOutputDir(apiclient.getCurrentPentest())
        path = apiclient.getResult(self.controller.getDbId(), os.path.join(outputDir,path))
        dialog.destroy()
        if path is not None:
            if os.path.isfile(path):
                dialog = ChildDialogQuestion(self.appliViewFrame, "Download completed",
                                            "The file has been downloaded.\n Would you like to open it?", answers=["Open", "Open folder", "Cancel"])
                self.appliViewFrame.wait_window(dialog.app)
                if dialog.rvalue == "Open":
                    utils.openPathForUser(path)
                    return
                elif dialog.rvalue == "Open folder":
                    utils.openPathForUser(os.path.dirname(path))
                    return
                else:
                    return
            path = None
        if path is None:
            tkinter.messagebox.showerror(
                "Download failed", "the file does not exist on sftp server")

    def createDefectCallback(self, _event=None):
        """Callback for tool click 
        Creates an empty defect view and open it's insert window with notes = tools notes.
        """
        modelData = self.controller.getData()
        toExport = modelData["notes"]
        for widget in self.appliViewFrame.winfo_children():
            widget.destroy()
        modelData["target_type"] = self.controller.model.__class__.__name__.lower()
        modelData["target_id"] = self.controller.model.getId()
        dv = DefectView(self.appliTw, self.appliViewFrame,
                        self.mainApp, DefectController(Defect(modelData)))
        dv.openInsertWindow(toExport)

    def showAssociatedCommand(self, _event=None):
        if self.appliTw is not None:
            self.appliTw.showItem(self.controller.getData()["command_iid"])
            

    def localLaunchCallback(self, _event=None):
        """
        Callback for the launch tool button. Will launch it on localhost pseudo 'worker'. 

        Args:
            event: Automatically generated with a button Callback, not used.
        """
        self.mainApp.scanManager.launchTask(self.controller.model,)
       
        self.form.clear()
        for widget in self.appliViewFrame.winfo_children():
            widget.destroy()
        self.openModifyWindow()

    def safeLaunchCallback(self, _event=None):
        """
        Callback for the launch tool button. Will queue this tool to a worker. 
        Args:
            event: Automatically generated with a button Callback, not used.
        Returns:
            None if failed. 
        """
        apiclient = APIClient.getInstance()
        return apiclient.sendQueueTasks(self.controller.model.getId()) is not None

    def launchCallback(self, _event=None):
        """
        Callback for the launch tool button. Will queue this tool to a worker. 
        Will try to launch respecting limits first. If it does not work, it will asks the user to force launch.

        Args:
            _event: Automatically generated with a button Callback, not used.
        """
        res = self.safeLaunchCallback()
        if res:
            self.form.clear()
            for widget in self.appliViewFrame.winfo_children():
                widget.destroy()
            self.openModifyWindow()

    def remoteInteraction(self, _event=None):
        self.mainApp.open_ro_terminal(self.controller.model.check_iid, self.title, self.controller, self.mainApp.scanManager)
        #dialog.app.wait_window(dialog.app)
        
    def stopCallback(self, _event=None):
        """
        Callback for the launch tool stop button. Will stop this task. 

        Args:
            _event: Automatically generated with a button Callback, not used.
        """
        apiclient = APIClient.getInstance()
        success = False
        success_local = self.mainApp.scanManager.stopTask(self.controller.model.getId())
        if success_local:
            apiclient.sendStopTask(self.controller.model.getId(), True) # send reset tool 
            success = True
        else:
            success_distant = apiclient.sendStopTask(self.controller.model.getId())
            delete_anyway = False
            if success_distant != True: # it can be a tuple so this check is invalid if False and Tuple
                delete_anyway = tkinter.messagebox.askyesno(
                    "Stop failed", """This tool cannot be stopped because its trace has been lost (The application has been restarted and the tool is still not finished).\n
                        Reset tool anyway?""")
            if delete_anyway:
                success = apiclient.sendStopTask(self.controller.model.getId(), True)
        if success:
            self.form.clear()
            for widget in self.appliViewFrame.winfo_children():
                widget.destroy()
            self.openModifyWindow()

    def resetCallback(self, reload=True,_event=None):
        """
        Callback for the reset tool stop button. Will reset the tool to a ready state. 

        Args:
            event: Automatically generated with a button Callback, not used.
        """
        self.controller.markAsNotDone()
        if reload:
            self.reopenView()

    @classmethod
    def DbToTreeviewListId(cls, parent_db_id):
        """Converts a mongo Id to a unique string identifying a list of tools given its parent
        Args:
            parent_db_id: the parent node mongo ID
        Returns:
            A string that should be unique to describe the parent list of tool node
        """
        if parent_db_id is None:
            return ""
        return str(parent_db_id)

    @classmethod
    def treeviewListIdToDb(cls, treeview_id):
        """Extract from the unique string identifying a list of tools the parent db ID
        Args:
            treeview_id: the treeview node id of a list of tools node
        Returns:
            the parent object mongo id as string
        """
        return str(treeview_id)

    def updateReceived(self, obj=None, old_obj=None):
        """Called when a tool update is received by notification.
        Update the tool treeview item (resulting in icon reloading)
        """
        try:
            self.appliTw.item(str(self.controller.getDbId()), text=str(
                self.controller.getModelRepr()), image=self.getIcon())
        except tk.TclError:
            pass
            #print("WARNING: Update received for a non existing tool "+str(self.controller.getModelRepr()))
        super().updateReceived()



"""Help the user to create a new pentest database.
"""
import tkinter as tk
from tkinter import ttk
from customtkinter import *
from pollenisatorgui.core.application.dialogs.ChildDialogAskText import ChildDialogAskText
from pollenisatorgui.core.application.scrollableframexplateform import ScrollableFrameXPlateform
from pollenisatorgui.core.application.scrollabletreeview import ScrollableTreeview
import pollenisatorgui.core.components.utils as utils
from PIL import Image, ImageTk

from pollenisatorgui.core.forms.formpanel import FormPanel

class ChildDialogToolsInstalled(CTkToplevel):
    """
    Open a child dialog of a tkinter application to show binary installed
    """
    cvalid_icon = None
    cbad_icon = None

    def validIcon(self):
        """Returns a icon indicating a valid state.
        Returns:
            ImageTk PhotoImage"""
        if self.__class__.cvalid_icon is None:
            self.__class__.cvalid_icon = ImageTk.PhotoImage(Image.open(utils.getValidMarkIconPath()))
        return self.__class__.cvalid_icon

    def badIcon(self):
        """Returns a icon indicating a bad state.
        Returns:
            ImageTk PhotoImage"""
        if self.__class__.cbad_icon is None:
            self.__class__.cbad_icon = ImageTk.PhotoImage(Image.open(utils.getBadMarkIconPath()))
        return self.__class__.cbad_icon

    
    def __init__(self, tools_results, parent=None):
        """
        Open a child dialog of a tkinter application to ask details about
        the new pentest.

        Args:
            parent: the tkinter parent view to use for this window construction.
        """
        super().__init__(parent)
        self.parent = parent
        self.tools_results = tools_results
        self.title("Tools configured supported")
        self.attributes("-type", "dialog")
        self.resizable(True, True)
        self.rvalue = None
        
        self.bind("<Escape>", self.quit)
        self.mainFrame = CTkFrame(self)
        self.reloadUI()
        self.mainFrame.pack(fill=tk.BOTH, ipadx=10, ipady=10, expand=1)
        try:
            self.wait_visibility()
            self.transient(self.parent)
            self.focus_force()
            self.grab_set()
            self.lift()
        except tk.TclError:
            pass

    def OnDoubleClick(self, event):
        item = self.tv.identify("item", event.x, event.y)
        if item is None or item == '':
            return
        plugin_name = self.tv.item(item)["text"]
        current_bin_path = self.tv.item(item)["values"][0]
        dialog_ask_text = ChildDialogAskText(self, info=f"New path for plugin {plugin_name}", default=current_bin_path ,multiline=False)
        self.wait_window(dialog_ask_text.app)
        if dialog_ask_text.rvalue is not None:
            new_path = dialog_ask_text.rvalue
            if utils.which_expand_alias(new_path):
                self.tv.item(item, values=(new_path,), image=self.validIcon())
            else:
                self.tv.item(item, values=(new_path,), image=self.badIcon())
                tk.messagebox.showerror("Invalid path", f"Path '{new_path}' does not seem to be detected as a valid binary.", parent=self)
    
    def reloadUI(self):
        for widget in self.mainFrame.winfo_children():
            widget.destroy()
        self.tv = ScrollableTreeview(self.mainFrame,('Plugin name','Local command Configured'), height=25, maxPerPage=25)
        self.tv.pack(side=tk.TOP, padx=10, pady=10, fill=tk.BOTH, expand=1)
        self.tv.bind("<Double-Button-1>", self.OnDoubleClick)
        for tool_result in self.tools_results.get("failures", []):
            self.tv.insert("", "end", tool_result["plugin"]["plugin"], text=tool_result["plugin"]["plugin"], values=(tool_result["bin_path"],),
                           image=self.badIcon())
        for tool_result in self.tools_results.get("successes", []):
            self.tv.insert("", "end", tool_result["plugin"]["plugin"], text=tool_result["plugin"]["plugin"], values=(tool_result["bin_path"],),
                           image=self.validIcon())
        CTkButton(self.mainFrame, text="OK", command=self.onOk).pack(side=tk.RIGHT)
            
        self.button = CTkButton(self.mainFrame, text="Cancel",command=self.onError,
                               fg_color=utils.getBackgroundColor(), text_color=utils.getTextColor(),
                               border_width=1, border_color="firebrick1", hover_color="tomato").pack(side=tk.RIGHT)
        self.mainFrame.pack(fill=tk.BOTH, ipadx=10, ipady=10, expand=1)
        try:
            self.wait_visibility()
            self.transient(self.parent)
            self.focus_force()
            self.grab_set()
            self.lift()
        except tk.TclError:
            pass
    
    def onError(self, _event=None):
        self.rvalue = None
        self.destroy()

    def onOk(self, _event=None):
        """
        Called when the user clicked the validation button.
        launch parsing with selected parser on selected file/directory.
        Close the window.

        Args:
            _event: not used but mandatory
        """
        res = {}
        for item in self.tv.infos:
            command_name = item
            line_info = self.tv.infos[item]
            res[command_name] = line_info["values"][0]
        self.rvalue = res
        self.destroy()
        return


"""Element parent Model. Common ground for every model"""

from pollenisatorgui.core.components.apiclient import APIClient
from pollenisatorgui.core.models.metaelement import MetaElement
from bson.objectid import ObjectId


class Element(metaclass=MetaElement):
    """
    Parent element for all model. This class should only be inherited.

    Attributes:
        coll_name:  collection name in pollenisator database
    """
    coll_name = None

    def __init__(self, _id, parent, infos):
        """
        Constructor to be inherited. Child model will all use this constructor.

        Args:
            _id: mongo database id
            parent: a parent mongo id object for this model.
            infos: a dicitonnary of custom information 
        """
        # Initiate a cachedIcon for a model, not a class.
        self._id = _id
        self.parent = parent
        self.infos = infos
        self.cachedIcon = None

    @classmethod
    def fetchObject(cls, pipeline):
        """Fetch one element from database and return the object 
        Args:
            pipeline: a Mongo search pipeline (dict)
        Returns:
            Returns the model object or None if nothing matches the pipeline.
        """
        apiclient = APIClient.getInstance()
        d = apiclient.find(cls.coll_name, pipeline, False)
        if d is None:
            return d
        # disabling this error as it is an abstract function
        return cls(d)  #  pylint: disable=no-value-for-parameter

    @classmethod
    def fetchPentestObjects(cls):
        apiclient = APIClient.getInstance()
        ds = apiclient.find(cls.coll_name, {}, True)
        if ds is None:
            return None
        for d in ds:
            # disabling this error as it is an abstract function
            yield cls(d)  


    @classmethod
    def fetchObjects(cls, pipeline):
        """Fetch many commands from database and return a Cursor to iterate over model objects
        Args:
            pipeline: a Mongo search pipeline (dict)
        Returns:
            Returns a cursor to iterate on model objects
        """
        apiclient = APIClient.getInstance()
        ds = apiclient.find(cls.coll_name, pipeline, True)
        if ds is None:
            return None
        for d in ds:
            # disabling this error as it is an abstract function
            yield cls(d)  #  pylint: disable=no-value-for-parameter

    def getId(self):
        """Returns the mongo id  of this element.
        Returns:
            bson.objectid.ObjectId
        """
        return self._id

    def getChecks(self):
        """Return wave assigned checks as a list of checkInstance objects
        Returns:
            list of CheckInstance objects
        """
        from pollenisatorgui.core.models.checkinstance import CheckInstance
        return CheckInstance.fetchObjects({"target_iid": str(self._id)})

    def getParentId(self):
        """Returns the mongo id  of this element parent.
        Returns:
            bson.objectid.ObjectId
        """
        if self.parent is None:
            try:
                self.parent = self._getParentId()  # pylint: disable=assignment-from-none
            except TypeError:
                return None
        return self.parent

    def _getParentId(self):
        """
        To be overriden
        Return the mongo ObjectId _id of the first parent of this object. 
        Returns:
            Returns the parent's ObjectId _id".
        Returns:
            None
        """
        return None

    def delete(self):
        """
        To be overriden
        Delete the object represented by this model in database.
        """
        # pass

    def addInDb(self):
        """
        To be overriden
        Add this model to pollenisator database.
        Returns: a tuple with :
                * bool for success
                * mongo ObjectId : already existing object if duplicate, create object id otherwise 
        """
        # pass

    def update(self, _pipeline_set=None):
        """
        To be overriden
        Update this object in database.
        Args:
            pipeline_set: (Opt.) A dictionnary with custom values. If None (default) use model attributes.
        """
        # pass




    def getTagsGroups(self):
        """Returns groups of tags that may not be applied at the same time
        Returns:
            List of list of strings
        """
        import pollenisatorgui.core.components.settings as settings

        tags = settings.Settings.getTags()
        return [tags, ["hidden"]]

    

    def getDetailedString(self):
        """To be inherited and overriden
        Returns a detailed string describing this element. Calls __str__ of children by default.
        Returns:
            string
        """
        return str(self)

    def updateInfos(self, newInfos):
        """Change all infos stores in self.infos with the given new ones and update database.
        Args:
            newInfos: A new dictionnary of custom information
        """
        if "" in newInfos:
            del newInfos[""]
        self.infos.update(newInfos)
        self.update()



"""View for share object. Handle node in treeview and present forms to user when interacted with."""

from pollenisatorgui.core.components.apiclient import APIClient
from pollenisatorgui.core.views.viewelement import ViewElement
from tkinter import TclError
import tkinter as tk

class ShareView(ViewElement):
    """View for share object. Handle node in treeview and present forms to user when interacted with.
    Attributes:
        icon: icon name to show in treeview. Icon filename must be in icon directory.
    """

    icon = 'share.png'
    def openModifyWindow(self, **kwargs):
        """
        Creates a tkinter form using Forms classes. This form aims to update or delete an existing Share
        """
        self.form.clear()
        share_data = self.controller.getData()
        panel = self.form
        panel_info = panel.addFormPanel(grid=True)
        panel_info.addFormLabel("IP")
        panel_info.addFormStr("IP", "", share_data.get("ip", ""), status="readonly", row=0, column=1)
        panel_info.addFormLabel("Share", row=1)
        panel_info.addFormStr("Share", "", share_data.get("share", ""), status="readonly", row=1, column=1)
        files = share_data.get('files', []) 
        if files is None:
            files = []
        defaults = []
        for file in files:
            defaults.append([file["path"],file["flagged"], file["size"]])
        self.tree = panel.addFormTreevw("Files", ("Path","Flagged","Size"), default_values=defaults, side="top")
        contextual_menus = {}
        for module in self.mainApp.modules:
            if module.get("name", "") == "Active Directory":
                contextual_menus |= module["object"].getAdditionalContextualCommands("share", self.tree, self)
        
        self.completeModifyWindow(editable=False)
        for cmdname, cmd in contextual_menus.items():
            self.tree.addContextMenuCommand(cmdname, cmd)
        
    def openInsertWindow(self):
        """
        Creates a tkinter form using Forms classes. This form aims to insert a new Share
        """
        # modelData = self.controller.getData()
        # top_panel = self.form.addFormPanel(grid=True)
        # top_panel.addFormLabel("Start date")
        # top_panel.addFormDate("Start date", self.mainApp, "01/01/2000 00:00:00", column=1)
        # top_panel.addFormLabel("End date", row=1)
        # top_panel.addFormDate(
        #     "End date", self.mainApp, "31/12/2099 00:00:00", row=1, column=1)
        # top_panel.addFormHelper(
        #     "Auto scan will only launch if current time fits this share", row=1, column=2)
        # # added in getEmptyModel
        # self.form.addFormHidden("waveName", modelData["wave"])

        # self.completeInsertWindow()
        raise NotImplementedError("ShareView.openInsertWindow not implemented")

    def getAdditionalContextualCommands(self):
        return {}
       
    
    @classmethod
    def DbToTreeviewListId(cls, parent_db_id):
        """Converts a mongo Id to a unique string identifying a list of shares given its parent
        Args:
            parent_db_id: the parent node mongo ID
        Returns:
            A string that should be unique to describe the parent list of defect node
        """
        return str(parent_db_id)+"|Shares"
    
    @classmethod
    def treeviewListIdToDb(cls, treeviewId):
        """Extract from the unique string identifying a list of defects the parent db ID
        Args:
            treeviewId: the treeview node id of a list of defects node
        Returns:
            the parent object mongo id as string
        """
        return str(treeviewId).split("|")[0]

    def addInTreeview(self, parentNode=None, **kwargs):
        """Add this view in treeview. Also stores infos in application treeview.
        Args:
            parentNode: if None, will calculate the parent. If setted, forces the node to be inserted inside given parentNode.
            _addChildren: not used here
        """

        self.appliTw.views[str(self.controller.getDbId())] = {"view": self}
        tags = self.controller.getTags()
        parentDbId = parentNode
        if parentNode is None:
            parentNode = self.getParentNode()
        elif "shares" not in parentNode:
            parentNode = self.__class__.DbToTreeviewListId(parentDbId)
        try:
            self.appliTw.insert(
                self.controller.getParentId(), 0, parentNode, text="Shares", image=self.getClassIcon())
        except TclError as e:  #  trigger if tools list node already exist
            pass
        try:
            self.appliTw.insert(parentNode, "end", str(
                self.controller.getDbId()), text=str(self.controller.getModelRepr()), tags=tags, image=self.getClassIcon())
        except:
            pass
        if "hidden" in tags:
            self.hide("tags")
        if self.mainApp.settings.is_checklist_view():
            self.hide("checklist_view")

   


import sys
import shlex
import multiprocessing
from pollenisatorgui.core.components.logger_config import logger

def pollex():
    if len(sys.argv) <= 1:
        print("Usage : pollex [-v] <command to execute>")
        sys.exit(1)
    verbose = False
    if sys.argv[1] == "-v":
        verbose = True
        execCmd = shlex.join(sys.argv[2:])
    else:
        execCmd = shlex.join(sys.argv[1:])
    pollex_exec(execCmd, verbose)

def pollex_exec(execCmd, verbose=False):
    """Send a command to execute for pollenisator-gui running instance
    """
    
    bin_name = shlex.split(execCmd)[0]
    if bin_name in ["echo", "print", "vim", "vi", "tmux", "nano", "code", "cd", "ls","pwd", "cat", "export"]:
        sys.exit(-1)
    import os
    import shutil
    import tempfile
    import time
    from pollenisatorgui.core.components.apiclient import APIClient
    from pollenisatorgui.pollenisator import consoleConnect, parseDefaultTarget
    import pollenisatorgui.core.components.utils as utils

    cmdName = os.path.splitext(os.path.basename(execCmd.split(" ")[0]))[0]
    apiclient = APIClient.getInstance()
    apiclient.tryConnection()
    res = apiclient.tryAuth()
    if not res:
        consoleConnect()
    res = apiclient.getDesiredOutputForPlugin(execCmd, "auto-detect")
    (success, data) = res
    if not success:
        print(msg)
        consoleConnect()
    res = apiclient.getDesiredOutputForPlugin(execCmd, "auto-detect")
    (success, data) = res
    if not success:
        print(msg)
        return
    cmdName +="::"+str(time.time()).replace(" ","-")
    default_target = parseDefaultTarget(os.environ.get("POLLENISATOR_DEFAULT_TARGET", ""))
    if default_target.get("tool_iid") is not  None:
        apiclient.setToolStatus(default_target.get("tool_iid"), ["running"])
    
    if not success:
        print("ERROR : "+data)
        return
    if not data:
        print("ERROR : An error as occured : "+str(data))
        return
    comm = data["command_line_options"]
    plugin_results = data["plugin_results"]
    if (verbose):
        print("INFO : Matching plugins are "+str(data["plugin_results"]))
    
    tmpdirname = tempfile.mkdtemp() ### HACK: tempfile.TemporaryDirectory() gets deleted early because a fork occurs in execute and atexit triggers.
    for plugin,ext in plugin_results.items():
        outputFilePath = os.path.join(tmpdirname, cmdName) + ext
        comm = comm.replace(f"|{plugin}.outputDir|", outputFilePath)
    if (verbose):
        print("Executing command : "+str(comm))
        print("output should be in "+str(outputFilePath))
    queue = multiprocessing.Queue()
    queueResponse = multiprocessing.Queue()
    #if comm.startswith("sudo "):
    #    returncode = utils.execute_no_fork(comm, None, True, queue, queueResponse, cwd=tmpdirname)
    #else:
    try:
        returncode = utils.execute(comm, None, queue, queueResponse, cwd=tmpdirname, printStdout=True)
    except KeyboardInterrupt:
        logger.debug("pollex KeyboardInterrupt for comm "+str(comm))
    except Exception as e:
        logger.debug("pollex Exception for comm "+str(comm)+" "+str(e))
    queue.put("kill", block=False)
    if len(plugin_results) == 1 and "Default" in plugin_results:
        if (verbose):
            print("INFO : Only default plugin found")
        response = input("No plugin matched, do you want to use default plugin to log the command and stdout ? (Y/n) :")
        if str(response).strip().lower() == "n":
            shutil.rmtree(tmpdirname)
            return
    logger.debug("pollex detect plugins "+str(plugin_results))
    atLeastOne = False
    error = ""
    for plugin,ext in plugin_results.items():
        outputFilePath = os.path.join(tmpdirname, cmdName) + ext
        if not os.path.exists(outputFilePath):
            if os.path.exists(outputFilePath+ext):
                outputFilePath+=ext
            else:
                print(f"ERROR : Expected file was not generated {outputFilePath}")
                error = "ERROR : Expected file was not generated"
                continue
        print(f"INFO : Uploading results {outputFilePath}")
        msg = apiclient.importExistingResultFile(outputFilePath, plugin, default_target, comm)
        print(msg)
        atLeastOne = True
    if not atLeastOne:
        notes = b""
        while not queueResponse.empty():
            q = queueResponse.get(block=False)
            if q:
                if isinstance(q, str):
                    notes += q.encode()
        apiclient.setToolStatus(default_target.get("tool_iid"), ["error"], error+"\nSTDOUT:\n"+notes.decode())
    shutil.rmtree(tmpdirname)



"""Widget with an entry to type a file path and a '...' button to pick from file explorer."""

import tkinter as tk
import tkinter.ttk as ttk
from customtkinter import *
from pollenisatorgui.core.forms.form import Form
import pollenisatorgui.core.components.utils as utils
import tkinter.filedialog
import os

class FormFile(Form):
    """
    Form field representing a path input.
    Default setted values: 
        state="readonly"
        if pack : padx = pady = 5, side = "right"
        if grid: row = column = 0 sticky = "west"
        entry "width"=  20
    Additional values to kwargs:
        modes: either "file" or "directory" to choose which type of path picker to open
    """

    def __init__(self, name, regexValidation="", default=[], **kwargs):
        """
        Constructor for a form file

        Args:
            name: the entry name (id).
            regexValidation: a regex used to check the input in the checkForm function., default is ""
            default: a default value for the Entry, default is ""
            kwargs: same keyword args as you would give to CTkFrame + "modes" which is either "file" or "directory" 
                    to choose which type of path picker to open
        """
        super().__init__(name)
        self.regexValidation = regexValidation
        self.default = default
        self.kwargs = kwargs
        self.listbox = None

    def constructView(self, parent):
        """
        Create the string view inside the parent view given

        Args:
            parent: parent FormPanel.
        """
        self.val = tk.StringVar()
        frame = CTkFrame(parent.panel, height=0)
        listboxframe = ttk.Frame(frame, height=0)
        listboxframe.grid_columnconfigure(0, weight=1)
        listboxframe.grid_rowconfigure(0, weight=1)
        self.callback = self.getKw("command", None)
        self.listbox = tk.Listbox(listboxframe, 
                              width=self.getKw("width", 50), height=self.getKw("height", 10), selectmode=tk.SINGLE, bg=utils.getBackgroundColor(), fg=utils.getTextColor())
        self.listbox.register_drop_target("*")
        self.listbox.bind('<<Drop:File>>', self.add_path_listbox)
        self.listbox.bind('<Delete>', self.delete_path_listbox)
        self.scrolbar = CTkScrollbar(
            listboxframe,
            orientation=tk.VERTICAL,
            height=0
        )
        self.scrolbarH = CTkScrollbar(
            listboxframe,
            orientation=tk.HORIZONTAL
        )
        self.listbox.grid(row=0, column=0, sticky=tk.NSEW)
        self.scrolbar.grid(row=0, column=1, sticky=tk.NS)
        self.scrolbarH.grid(row=1, column=0, sticky=tk.EW)
        # displays the content in listbox
        self.listbox.configure(yscrollcommand=self.scrolbar.set)
        self.listbox.configure(xscrollcommand=self.scrolbarH.set)

        # view the content vertically using scrollbar
        self.scrolbar.configure(command=self.listbox.yview)
        self.scrolbarH.configure(command=self.listbox.xview)
        self.add_paths(self.default)
        listboxframe.pack(expand=0, fill=tk.X, side=tk.TOP, anchor=tk.CENTER)
        self.modes = self.getKw("mode", "file").split("|")
        btn_frame = CTkFrame(frame,height=0)
        info = CTkLabel(btn_frame, text="Or Drag and Drop")
        info.pack(side="right", pady=5,padx=5)
        if "file" in self.modes:
            text = self.getKw("text", "Add file")
            search_btn = CTkButton(
                btn_frame, text=text, command=lambda :self.on_click(None, parent))
            search_btn.pack(side="right", pady=5)
        if "directory" in self.modes:
            text = self.getKw("text", "Add directory")
            search_btn = CTkButton(
                btn_frame, text=text, command=lambda :self.on_click_dir(None, parent))
            search_btn.pack(side="right", pady=5)
        
        btn_frame.pack(side=tk.BOTTOM, anchor=tk.CENTER)
        if parent.gridLayout:
            frame.grid(row=self.getKw("row", 0),
                       column=self.getKw("column", 0), **self.kwargs)
        else:
            frame.pack(fill=self.getKw("fill", "x"), side=self.getKw(
                "side", "top"), anchor=self.getKw("anchor", "center"), pady=self.getKw("pady", 5), padx=self.getKw("padx", 10), **self.kwargs)

    def on_click(self, _event=None, parent=None):
        """Callback when '...' is clicked and modes Open a file selector (tkinter.filedialog.askopenfilename)
        Args:
            _event: not used but mandatory
        Returns:
            None if no file name is picked,
            the selected file full path otherwise.
        """
        
        f = tkinter.filedialog.askopenfilename(title="Select a file", parent=parent.panel)
        if f is None or not isinstance(f, str):  # asksaveasfile return `None` if dialog closed with "cancel".
            return
        if f != "":
            filename = str(f)
            self.add_paths([filename])
            if callable(self.callback):
                self.callback()

    def on_click_dir(self, _event=None, parent=None):
        """Callback when '...' is clicked and modes="directory" was set.
        Open a directory selector (tkinter.filedialog.askdirectory)
        Args:
            _event: not used but mandatory
        Returns:
            None if no directory is picked,
            the selected directory full path otherwise.
        """

        f = tkinter.filedialog.askdirectory(title="Select a directory", parent=parent.panel)
        if f is None or not isinstance(f, str):  # asksaveasfile return `None` if dialog closed with "cancel".
            return
        if f != "":
            filename = str(f)
            self.add_paths([filename])
            if callable(self.callback):
                self.callback()

    def getValue(self):
        """
        Return the form value. Required for a form.

        Returns:
            Return the entry value as string.
        """
        paths = self.get_paths()
        print(paths)
        return paths

    def checkForm(self):    
        """
        Check if this form is correctly filled. Check with the regex validation given in constructor.

        Returns:
            {
                "correct": True if the form is correctly filled, False otherwise.
                "msg": A message indicating what is not correctly filled.
            }
        """
        import re
        values = self.getValue()
        if not values and self.kwargs.get("required", False):
            return False, f"this form must is required {self.name}"
        for value in values:
            if re.match(self.regexValidation,value) is None:
                return False, f"{value} is incorrect"
        return True, ""

    def setFocus(self):
        """Set the focus to the ttk entry part of the widget.
        """
        self.listbox.focus_set()

    def add_paths(self, paths):
        for path in paths:
            self.listbox.insert("end", path)
    
    def get_paths(self):
        return list(self.listbox.get('@1,0', tk.END))

    def add_path_listbox(self, event):
        data = utils.drop_file_event_parser(event)
        added = []
        for d in data:
            if os.path.isfile(d) and "file" in self.modes:
                added.append(d)
            elif os.path.isdir(d) and "directory" in self.modes:
                added.append(d)
        if added:
            self.add_paths(added)
        if self.callback and callable(self.callback):
            self.callback()
        
        
    def delete_path_listbox(self, event):
        curr = self.listbox.curselection()
        for i in curr:
            self.listbox.delete(i)
        if callable(self.callback):
            self.callback()


"""Remark Model."""

import os
from bson.objectid import ObjectId
from pollenisatorgui.core.models.element import Element
from pollenisatorgui.core.components.apiclient import APIClient
import json

class Remark(Element):
    """
    Represents a Remark object that defines a synthesis point. A Remark is a note added by a pentester on a report to complete the defect section.

    Attributes:
        coll_name: collection name in pollenisator database
    """
    coll_name = "remarks"

    def __init__(self, valuesFromDb=None):
        """Constructor
        Args:
            valueFromDb: a dict holding values to load into the object. A mongo fetched remark is optimal.
                        possible keys with default values are : _id (None), 
                        type("Positive|Negative|Neutral"), title("")
        """
        if valuesFromDb is None:
            valuesFromDb = {}
        super().__init__(valuesFromDb.get("_id", None), None, {})
        self.initialize(valuesFromDb.get("type", "Positive"), valuesFromDb.get("title", ""), valuesFromDb.get("description", ""))

    def initialize(self, typeof, title, description):
        """Set values of remark
        Args:
            type: the type of the remark (Positive, Neutral or Negative)
            title: a title for this remark
            description: a description
        Returns:
            this object
        """
        self.title = title
        self.type = typeof
        self.description = description
        return self

    def delete(self):
        """
        Delete the remark represented by this model in database.
        """
        ret = self._id
        apiclient = APIClient.getInstance()
        apiclient.deleteFromDb(apiclient.getCurrentPentest(),"remarks", {"_id": ObjectId(ret)})

    def addInDb(self):
        """
        Add this defect to pollenisator database.
        Returns: a tuple with :
                * bool for success
                * mongo ObjectId : already existing object if duplicate, create object id otherwise 
        """
        base = self.getDbKey()
        # add to base parameters defined or not depending on the lvl.
        # Checking unicity
        apiclient = APIClient.getInstance()
        existing = apiclient.findInDb(apiclient.getCurrentPentest(), "remarks", base, False)
        if existing is not None:
            return False, existing["_id"]

        # Those are added to base after tool's unicity verification

        base["type"] = self.type
        base["title"] = self.title
        base["description"] = self.description
        # Get parent for notifications
        res = apiclient.insertInDb(apiclient.getCurrentPentest(), "remarks", base, notify=True)
        self._id = ObjectId(res)
        return True, self._id

    def update(self, pipeline_set=None):
        """Update this object in database.
        Args:
            pipeline_set: (Opt.) A dictionnary with custom values. If None (default) use model attributes.
        """
        apiclient = APIClient.getInstance()
        if pipeline_set is None:
            apiclient.updateInDb(apiclient.getCurrentPentest(), "remarks", {"_id":ObjectId(self._id)}, {"$set":{"type": self.type, "title": self.title, "description": self.description}})
        else:
            apiclient.updateInDb(apiclient.getCurrentPentest(),  {"_id":ObjectId(self._id)}, ObjectId(self._id), pipeline_set)

    def getData(self):
        """Return defect attributes as a dictionnary matching Mongo stored defects
        Returns:
            dict with keys title, ease, ipact, risk, redactor, type, notes, ip, port, proto, proofs, _id, tags, infos
        """
        return {"title": self.title, "type": self.type, "description": self.description}


    def _getParent(self):
        """
        Return the mongo ObjectId _id of the first parent of this object. For a Defect it is either an ip or a port depending on the Defect's level.

        Returns:
            Returns the parent's ObjectId _id".
        """
        return None
    
    def getParent(self):
        return None


    def __str__(self):
        """
        Get a string representation of a defect.

        Returns:
            Returns the defect +title.
        """
        return self.title

    def getDbKey(self):
        """Return a dict from model to use as unique composed key.
        Returns:
            A dict (1 key : "title")
        """
        return {"title": self.title}




# -*- coding: utf-8 -*-
from pollenisatorgui.core.components.apiclient import APIClient
from bson import ObjectId
from pollenisatorgui.core.application.dialogs.ChildDialogAskText import ChildDialogAskText
from neo4j import GraphDatabase

def mark_as_owned(uri, users, username, password):
    driver = GraphDatabase.driver(uri, auth=(username, password))
    with driver.session() as session:
        for user in users:
            account = user.get("username", "")
            if account == "":
                continue
            domain = user.get("domain", "")
            if domain == "":
                continue
            the_query = "MATCH (n) WHERE (n.name = \"{}@{}\") SET n.owned = true".format(account.strip().upper(), domain.upper())
            graph = session.run(the_query)
    driver.close()

def main(apiclient, appli, **kwargs):
    APIClient.setInstance(apiclient)
    users = apiclient.find("users", {"type":"user", "password":{"$ne":""}}, True)
    if users is None or not users:
        return False, "No owned users found"
    dialog = ChildDialogAskText(appli, "Bloodhound uri:", "bolt://localhost:7687", False)
    appli.wait_window(dialog.app)
    if dialog.rvalue is None:
        return False, "No URI given"
    bloodhound_uri = dialog.rvalue
    dialog = ChildDialogAskText(appli, "neo4j username:", "neo4j", False)
    appli.wait_window(dialog.app)
    if dialog.rvalue is None:
        return False, "No username given"
    bloodhound_username = dialog.rvalue
    dialog = ChildDialogAskText(appli, "neo4j password:", "neo4j", False, secret=True)
    appli.wait_window(dialog.app)
    if dialog.rvalue is None:
        return False, "No password given"
    bloodhound_password = dialog.rvalue
    mark_as_owned(bloodhound_uri, users, bloodhound_username, bloodhound_password)
    return True, f"Marked {len(users)} users as owned in bloodhound"



"""ChildDialogExportSelection class
Ask the user to select fields and returns the selected options"""
import tkinter as tk
import tkinter.ttk as ttk
from customtkinter import *
from pollenisatorgui.core.forms.formpanel import FormPanel
from pollenisatorgui.core.views.viewelement import ViewElement


class ChildDialogExportSelection:
    """
    Open a child dialog of a tkinter application to ask the user to select fields between many.
    """

    def __init__(self, parent, keys):
        """
        Open a child dialog of a tkinter application to ask details about
        an export of treeview items.

        Args:
            parent: the tkinter parent view to use for this window construction.
            keys: The keys to export
        """
        self.rvalue = None
        self.parent = parent
        self.app = CTkToplevel(parent)
        self.app.attributes("-type", "dialog")
        self.app.bind("<Escape>", self.onError)
        self.app.title("Export selection")
        appFrame = CTkFrame(self.app)
        self.form = FormPanel()
        self.form.addFormChecklist("Fields", sorted(keys), [])
        self.form.addFormButton("Export", self.onOk)
        self.rvalue = None
        self.form.constructView(appFrame)
        appFrame.pack(ipadx=10, ipady=10)
        try:
            self.app.wait_visibility()
            self.app.transient(parent)
            self.app.focus_force()
            self.app.grab_set()
            self.app.lift()
        except tk.TclError:
            pass

    def onError(self, event=None):
        self.app.destroy()
        self.rvalue = None
        return None
    
    def onOk(self, _event=None):
        """Called the the Export button is pressed.
        return a list of strings corresponding to the selected fields.
        
        Args:
            _event: not used but mandatory"""
        res, msg = self.form.checkForm()
        if res:
            form_values = self.form.getValue()
            form_values_as_dicts = ViewElement.list_tuple_to_dict(form_values)
            mfields = form_values_as_dicts["Fields"]
            fields = [k for k, v in mfields.items() if v == 1]
            self.rvalue = fields
            self.app.destroy()
        else:
            tk.messagebox.showwarning(
                "Form not validated", msg, parent=self.app)



import pollenisatorgui.core.components.utils as utils
from pollenisatorgui.core.application.dialogs.ChildDialogQuestion import ChildDialogQuestion
from pollenisatorgui.core.application.dialogs.ChildDialogCombo import ChildDialogCombo
from pollenisatorgui.core.application.dialogs.ChildDialogAskText import ChildDialogAskText
import os
from pollenisatorgui.core.components.apiclient import APIClient
import psutil

from pollenisatorgui.scripts.lan.utils import checkPath


def main(apiclient, appli, **kwargs):
    res, path = checkPath(["mitm6", "mitm6.py"])
    if not res:
        return False, path
    res, path = checkPath(["ntlmrelayx", "ntlmrelayx.py"])
    if not res:
        return False, path
    APIClient.setInstance(apiclient)
    smb_signing_list = apiclient.find("computers", {"infos.signing":"False"}, True)
    export_dir = utils.getExportDir()
    file_name = os.path.join(export_dir, "relay_list.lst")
    domains = set()
    liste = []
    with open(file_name, "w") as f:
        for computer in smb_signing_list:
            ip = computer.get("ip", "")
            domain = computer.get("domain", "")
            if domain.strip() != "":
                domains.add(domain)

            if ip != "":
                liste.append(ip)
                f.write(ip+"\n")
    if len(liste) == 0:
        return False, "No relayable host found yet"
    # 
    relaying_loot_path = os.path.join(export_dir, "loot_relay")
    try:
        os.makedirs(relaying_loot_path)
    except:
        pass
    relaying_loot_path = os.path.join(relaying_loot_path, "hashes-mitm6.log")
    addrs = psutil.net_if_addrs()
    dialog = ChildDialogCombo(None, addrs.keys(), displayMsg="Choose your ethernet device to listen on")
    dialog.app.wait_window(dialog.app)
    device = dialog.rvalue
    if device is None:
        return False, "No device selected"
    domain = ""
    if len(domains) == 0:
        dialog = ChildDialogAskText(None, "Enter domain name", multiline=False)
        dialog.app.wait_window(dialog.app)
        domain = dialog.rvalue
    elif len(domains) == 1:
        domain = domains[0]
    else:
        dialog = ChildDialogCombo(None, domains, displayMsg="Choose target domain")
        dialog.app.wait_window(dialog.app)
        domain = dialog.rvalue
    if domain is None:
        return False, "No domain choosen"

    address = addrs[device][0].address
    cmd = f"{path} -tf {file_name} -6 -wh {address} -of {relaying_loot_path}/"
    if os.geteuid() != 0:
        cmd = "sudo "+cmd
    appli.launch_in_terminal(kwargs.get("default_target",None), "ntlmrelayx for mitm6", f"{cmd}", use_pollex=False)
    cmd = f"mitm6 -i {device} -d {domain}"
    if os.geteuid() != 0:
        cmd = "sudo "+cmd
    appli.launch_in_terminal(kwargs.get("default_target",None), "mitm6", f"{cmd}", use_pollex=False)
    return True, f"Listening ntlmrelayx with mittm6 opened, loot directory is here:"+str(relaying_loot_path)+"\n"
    



"""Describe tkinter Entry with default common args"""

import tkinter as tk
import tkinter.ttk as ttk
from customtkinter import *
from pollenisatorgui.core.forms.form import Form
import pyperclip
import pollenisatorgui.core.components.utils as utils
from pollenisatorgui.core.application.pollenisatorentry import PopoEntry

class FormStr(Form):
    """
    Form field representing a string input.
    Default setted values:
        width=20
        if pack : padx = pady = 5, side = left
        if grid: row = column = 0 sticky = "East"
    """

    def __init__(self, name, regexValidation="", default="", contextualMenu=None, **kwargs):
        """
        Constructor for a form entry

        Args:
            name: the entry name (id).
            regexValidation: a regex used to check the input in the checkForm function., default is ""
            default: a default value for the Entry, defauult is ""
            contextualMenu: (Opt.) a contextualMenu to open when right clicked. default is None
            kwargs: same keyword args as you would give to PopoEntry
                    + binds:  a dictionnary of tkinter binding with shortcut as key and callback as value
        """
        super().__init__(name)
        self.regexValidation = regexValidation
        self.default = default
        self.contextualMenu = contextualMenu
        self.kwargs = kwargs
        self.widgetMenuOpen = None
        self.entry = None

    def _initContextualMenu(self, parent):
        """Initialize the contextual menu for paperclip.
        Args:
            parent: the tkinter parent widget for the contextual menu
        """
        # FIXME Add to given menu instead of Overriding given contextual menu
        self.contextualMenu = utils.craftMenuWithStyle(parent)
        self.contextualMenu.add_command(
            label="Copy", command=self.copy)
        self.contextualMenu.add_command(
            label="Cut", command=self.cut)
        self.contextualMenu.add_command(
            label="Paste", command=self.paste)
        self.contextualMenu.add_command(
            label="Close", command=self.close)

    def close(self):
        """Option of the contextual menu : Close the contextual menu by doing nothing
        """
        pass

    def copy(self):
        """Option of the contextual menu : Copy entry text to clipboard
        """
        pyperclip.copy(self.entry.selection_get())

    def cut(self):
        """Option of the contextual menu : Cut entry text to clipboard
        """
        sel = self.entry.selection_get()
        if sel:
            pyperclip.copy(sel)
            self.entry.delete(tk.SEL_FIRST, tk.SEL_LAST)

    def paste(self, event=None):
        """Option of the contextual menu : Paste clipboard content to entry
        """
        if event is None:
            buff = pyperclip.paste()
        else:
            buff = event.widget.clipboard_get()
        if buff:
            try:
                self.entry.delete(tk.SEL_FIRST, tk.SEL_LAST)
            except tk.TclError:
                pass
            insert_index = self.entry.index(tk.INSERT)
            self.entry.insert(insert_index, buff)
        return "break"

    def popup(self, event):
        """
        Fill the self.widgetMenuOpen and reraise the event in the editing window contextual menu

        Args:
            event: a ttk Treeview event autofilled. Contains information on what treeview node was clicked.
        """
        self.widgetMenuOpen = event.widget
        self.contextualMenu.tk_popup(event.x_root, event.y_root)
        self.contextualMenu.focus_set()
        self.contextualMenu.bind('<FocusOut>', self.popupFocusOut)

    def popupFocusOut(self, _event=None):
        """Callback for focus out event. Destroy contextual menu
        Args:
            _event: not used but mandatory
        """
        self.contextualMenu.unpost()

    def constructView(self, parent):
        """
        Create the string view inside the parent view given

        Args:
            parent: parent FormPanel.
        """
        self.entry = PopoEntry(parent.panel, placeholder_text=self.getKw("placeholder", self.getKw("placeholder_text", None)), width=self.getKw(
            "width", 200), state=self.getKw("state", "normal"), show=self.getKw("show", None))
        self._initContextualMenu(self.entry)
        self.entry.bind("<<Paste>>", self.paste)
        for bind, bind_call in self.getKw("binds", {}).items():
            self.entry.bind(bind, bind_call)
        if self.default != "" and self.default is not None:
            self.setValue(self.default)
            

        if parent.gridLayout:
            self.entry.grid(row=self.getKw("row", 0), column=self.getKw(
                "column", 0), sticky=self.getKw("sticky", tk.W))
        else:
            self.entry.pack(side=self.getKw("side", "left"), padx=self.getKw(
                "padx", 10), pady=self.getKw("pady", 5), expand=self.getKw("expand", True), fill=self.getKw("fill", "x"))

 

    def getValue(self):
        """
        Return the form value. Required for a form.

        Returns:
            Return the entry value as string.
        """
        return self.entry.get()
    
    def setValue(self, newval):
        """
        Set the text value.
        Args:
            newval: the new value to be set inside the text
        """
        state = self.entry.cget("state")
        self.entry.configure(state="normal")
        self.entry.delete("0", "end")
        self.entry.insert("0", newval)
        self.entry.configure(state=state)

    def checkForm(self):
        """
        Check if this form is correctly filled. Check with the regex validation given in constructor.

        Returns:
            {
                "correct": True if the form is correctly filled, False otherwise.
                "msg": A message indicating what is not correctly filled.
            }
        """
        import re
        if re.match(self.regexValidation, self.getValue()) is not None:
            return True, ""
        return False, self.kwargs.get("error_msg", self.name+" value is incorrect. \n "+self.getValue()+"does not match regex "+str(self.regexValidation))

    def setFocus(self):
        """Set the focus to the ttk entry widget.
        """
        self.entry.focus_set()



"""This class pop a defect view form in a subdialog"""

import tkinter as tk
import tkinter.ttk as ttk
from customtkinter import *
from pollenisatorgui.core.application.pollenisatorentry import PopoEntry
import pollenisatorgui.core.components.utils as utils

class ChildDialogFixView:
    """
    Open a child dialog of a tkinter application to answer a question.
    """
    def __init__(self, parent, fixModel=None):
        """
        Open a child dialog of a tkinter application to choose autoscan settings.

        Args:
            parent: the tkinter parent view to use for this window construction.
            defectModel : A Defect Model object to load default values. None to have empty fields, default is None.
        """
        self.app = CTkToplevel(parent)
        self.app.attributes("-type", "dialog")
        self.app.title("Fix edition")
        self.app.resizable(True, True)
        self.app.bind("<Escape>", self.cancel)
        self.rvalue = None
        topFrame = CTkFrame(self.app)
        appFrame = CTkFrame(topFrame)
        if fixModel is None:
            fixModel = {"title":"Title", "gain": "Moderate", "execution": "Moderate"}

        title_lbl = CTkLabel(appFrame, text="Title")
        title_lbl.pack(side="left", padx = 3)
        self.title_entry = PopoEntry(appFrame, width=200)
        self.title_entry.pack(side="left",padx = 10)
        self.title_entry.insert(0, fixModel.get("title", ""))
        execution_lbl = CTkLabel(appFrame, text="Execution")
        execution_lbl.pack(side="left",padx = 3)
        self.execution_combo = CTkComboBox(appFrame, values=tuple(["Quick Win", "Moderate", "Hard"]),state="readonly")
        self.execution_combo.pack(side="left",padx = 10)
        self.execution_combo.set(fixModel.get("execution", "Moderate"))
        gain_lbl = CTkLabel(appFrame, text="Gain")
        gain_lbl.pack(side="left",padx = 3)
        self.gain_combo = CTkComboBox(appFrame, values=tuple(["Weak", "Mean", "Strong"]),state="readonly")
        self.gain_combo.pack(side="left",padx = 10)
        self.gain_combo.set(fixModel.get("gain", "Mean"))
        appFrame.pack(fill=tk.BOTH, ipady=10, ipadx=10, expand=True)
        detailedFrame = CTkFrame(topFrame)
        CTkLabel(detailedFrame, text="Synthesis").pack(fill="x",pady=5,padx=5)
        self.synthesis_txt = CTkTextbox(detailedFrame,wrap="word")
        self.synthesis_txt.pack(fill="x")
        self.synthesis_txt.insert(tk.INSERT, fixModel.get("synthesis", "Synthesis"))
        CTkLabel(detailedFrame, text="Description").pack(fill="x",pady=5,padx=5)
        self.description_txt = CTkTextbox(detailedFrame, wrap="word")
        self.description_txt.pack(fill="x")
        self.description_txt.insert(tk.INSERT, fixModel.get("description", "Description"))
        detailedFrame.pack(fill=tk.BOTH, ipady=10, ipadx=10, expand=True)
        btn_frame = CTkFrame(topFrame)
        cancel_button = CTkButton(btn_frame, text="Cancel", 
                               fg_color=utils.getBackgroundColor(), text_color=utils.getTextColor(),
                               border_width=1, border_color="firebrick1", hover_color="tomato")
        cancel_button.pack(side="left", padx=5, pady=10)
        cancel_button.bind('<Button-1>', self.cancel)
        ok_button = CTkButton(btn_frame, text="OK")
        ok_button.pack(side="left", padx=5, pady=10)
        ok_button.bind('<Button-1>', self.okCallback)
        topFrame.pack(side="top")
        btn_frame.pack(side="bottom", anchor="e")
        self.app.transient(parent)
        try:
            self.app.wait_visibility()
            self.app.transient(parent)
            self.app.grab_set()
            self.app.focus_force()
            self.app.lift()
        except tk.TclError:
            pass

    def cancel(self, _event=None):
        """called when canceling the window.
        Close the window and set rvalue to False
        Args:
            _event: Not used but mandatory"""
        self.rvalue = None
        self.app.destroy()

    def okCallback(self, _event=None):
        """called when pressing the validating button
        Close the window if the form is valid.
        Set rvalue to True and perform the defect update/insert if validated.
        Args:
            _event: Not used but mandatory"""
        
        title = self.title_entry.get().strip()
        gain = self.gain_combo.get()
        execution = self.execution_combo.get()
        synthesis = self.synthesis_txt.get('1.0', tk.END)
        description = self.description_txt.get('1.0', tk.END)
        self.rvalue = {"title":title, "gain":gain, "execution":execution, "synthesis":synthesis, "description":description}
        self.app.destroy()



"""View for multi selected object clicked. Present an multi modify form to user when interacted with."""

from tkinter import ttk
from pollenisatorgui.core.views.viewelement import ViewElement
from pollenisatorgui.core.components.settings import Settings
import pollenisatorgui.core.components.utils as utils
from customtkinter import *
from PIL import Image

class MultiSelectionView(ViewElement):
    """View for multi selected object clicked. Present an multi modify form to user when interacted with."""

    def __init__(self, appliTw, appViewFrame, mainApp):
        super().__init__(appliTw, appViewFrame, mainApp, None)

    def tagClicked(self, name):
        """Separate callback to apply when a tag button is clicked
        Applies the clicked tag to all selected objects
        Args:
            name: tag name clicked
        """
        return lambda : self.appliTw.setTagFromMenubar(name)

    def openModifyWindow(self, **kwargs):
        """
        Creates a tkinter form using Forms classes. This form aims to update or perform actions on multiple different objects common properties like tags.
        """
        self.form.clear()
        top_panel = self.form.addFormPanel()
        top_panel.addFormButton("Export", self.appliTw.exportSelection)
        top_panel.addFormButton("Hide", self.appliTw.hideSelection)
        self.delete_image = CTkImage(Image.open(utils.getIcon("delete.png")))
        #top_panel.addFormButton("Custom Command", self.appliTw.customCommand)
        top_panel.addFormButton("Delete", self.appliTw.deleteSelected, image=self.delete_image,
                               fg_color=utils.getBackgroundColor(), text_color=utils.getTextColor(),
                               border_width=1, border_color="firebrick1", hover_color="tomato")
        panTags = self.form.addFormPanel(grid=True)
        registeredTags = Settings.getTags()
        keys = list(registeredTags.keys())
        column = 0
        listOfLambdas = [self.tagClicked(keys[i]) for i in range(len(keys))]
        for registeredTag, tag_info in registeredTags.items():
            s = ttk.Style(self.mainApp)
            btn_tag = panTags.addFormButton(registeredTag, listOfLambdas[column], column=column)
            btn_tag.configure(fg_color=tag_info.get("color"))
            column += 1
        self.showForm()


"""ChildDialogEditPassword class
Ask the user to edit a user password"""
import tkinter as tk
import tkinter.ttk as ttk
from customtkinter import *
from pollenisatorgui.core.forms.formpanel import FormPanel
from pollenisatorgui.core.views.viewelement import ViewElement
from pollenisatorgui.core.components.apiclient import APIClient

class ChildDialogEditPassword:
    """
    Open a child dialog of a tkinter application to ask a user to reset its password.
    """

    def __init__(self, parent, username, askOldPwd=True):
        """
        Open a child dialog of a tkinter application to ask the new password and possibly the old

        Args:
            parent: the tkinter parent view to use for this window construction.
            username: The username to reset the password of
            askOldPwd : a boolean to use changePassword (user api) or resetPassword (admin api)
        """
        self.parent = parent
        self.app = CTkToplevel(parent)
        self.app.attributes("-type", "dialog")
        self.app.bind("<Escape>", self.onError)
        self.askOldPwd = askOldPwd
        self.app.title("Change "+str(username)+" password")
        appFrame = CTkFrame(self.app)
        self.form = FormPanel()
        self.form.addFormLabel("Username")
        self.form.addFormStr("Username", ".+", default=username, readonly=True)
        if askOldPwd:
            self.form.addFormLabel("Old password")
            self.form.addFormStr("Old password", ".+", show="*")
        self.form.addFormLabel("New Password")
        self.form.addFormStr("New password", ".{8,}", show="*", error_msg="New password must be at least 8 characters long")
        self.form.addFormButton("OK", self.onOk)
        self.rvalue = None
        self.form.constructView(appFrame)
        appFrame.pack(ipadx=10, ipady=10)
        try:
            self.app.wait_visibility()
            self.app.transient(parent)
            self.app.focus_force()
            self.app.grab_set()
            self.app.lift()
        except tk.TclError:
            pass

    def onError(self, _event=None):
        self.app.destroy()

    def onOk(self, _event=None):
        """Called the ok button is pressed.
        
        Args:
            _event: not used but mandatory"""
        res, msg = self.form.checkForm()
        apiclient = APIClient.getInstance()
        success = False
        if res:
            form_values = self.form.getValue()
            form_values_as_dicts = ViewElement.list_tuple_to_dict(form_values)
            username = form_values_as_dicts["Username"]
            newPwd = form_values_as_dicts["New password"]
            if self.askOldPwd:
                oldPwd = form_values_as_dicts["Old password"]
                msg = apiclient.changeUserPassword(oldPwd, newPwd)
            else:
                msg = apiclient.resetPassword(username, newPwd)
            if msg != "":
                tk.messagebox.showwarning(
                    "Change password", msg, parent=self.app)
        else:
            tk.messagebox.showwarning(
                "Form not validated", msg, parent=self.app)
        
        self.app.destroy()
        

